import React, { useEffect, useRef, useState } from "react";
import { useFrame } from "@react-three/fiber";
import * as THREE from "three";
import { Line } from "@react-three/drei";
import {
  useAnimationPlaySpeed,
  usePauseButtonStore,
  usePlayButtonStore,
  useResetButtonStore,
} from "../../../../../store/usePlayButtonStore";

function RoboticArmAnimator({
  HandleCallback,
  restPosition,
  ikSolver,
  targetBone,
  armBot,
  logStatus,
  path,
}: any) {
  const progressRef = useRef(0);
  const curveRef = useRef<THREE.Vector3[] | null>(null);
  const [currentPath, setCurrentPath] = useState<[number, number, number][]>(
    []
  );
  const [circlePoints, setCirclePoints] = useState<[number, number, number][]>(
    []
  );
  const [customCurvePoints, setCustomCurvePoints] = useState<
    THREE.Vector3[] | null
  >(null);

  // Zustand stores
  const { isPlaying } = usePlayButtonStore();
  const { isPaused } = usePauseButtonStore();
  const { isReset } = useResetButtonStore();
  const { speed } = useAnimationPlaySpeed();

  // Update path state whenever `path` prop changes
  useEffect(() => {
    setCurrentPath(path);
  }, [path]);

  // Reset logic when `isPlaying` changes
  useEffect(() => {
    if (!isPlaying) {
      setCurrentPath([]);
      curveRef.current = null;
    }
  }, [isPlaying]);

  // Handle circle points based on armBot position
  useEffect(() => {
    const points = generateRingPoints(1.6, 64);
    setCirclePoints(points);
  }, [armBot.position]);

  function generateRingPoints(radius: any, segments: any) {
    const points: [number, number, number][] = [];
    for (let i = 0; i < segments; i++) {
      // Calculate angle for current segment
      const angle = (i / segments) * Math.PI * 2;
      // Calculate x and z coordinates (y remains the same for a flat ring)
      const x = Math.cos(angle) * radius;
      const z = Math.sin(angle) * radius;
      points.push([x, 1.5, z]);
    }
    return points;
  }

  const findNearestIndex = (
    nearestPoint: [number, number, number],
    points: [number, number, number][],
    epsilon = 1e-6
  ) => {
    for (let i = 0; i < points.length; i++) {
      const [x, y, z] = points[i];
      if (
        Math.abs(x - nearestPoint[0]) < epsilon &&
        Math.abs(y - nearestPoint[1]) < epsilon &&
        Math.abs(z - nearestPoint[2]) < epsilon
      ) {
        return i; // Found the matching index
      }
    }
    return -1; // Not found
  };

  // Handle nearest points and final path (including arc points)
  useEffect(() => {
    if (circlePoints.length > 0 && currentPath.length > 0) {
      const start = currentPath[0];
      const end = currentPath[currentPath.length - 1];

      const raisedStart = [start[0], start[1] + 0.5, start[2]] as [
        number,
        number,
        number
      ];
      const raisedEnd = [end[0], end[1] + 0.5, end[2]] as [
        number,
        number,
        number
      ];

      const findNearest = (target: [number, number, number]) => {
        return circlePoints.reduce((nearest, point) => {
          const distance = Math.hypot(
            target[0] - point[0],
            target[1] - point[1],
            target[2] - point[2]
          );
          const nearestDistance = Math.hypot(
            target[0] - nearest[0],
            target[1] - nearest[1],
            target[2] - nearest[2]
          );
          return distance < nearestDistance ? point : nearest;
        }, circlePoints[0]);
      };

      const nearestToStart = findNearest(raisedStart);

      const nearestToEnd = findNearest(raisedEnd);

      const indexOfNearestStart = findNearestIndex(
        nearestToStart,
        circlePoints
      );

      const indexOfNearestEnd = findNearestIndex(nearestToEnd, circlePoints);

      // Find clockwise and counter-clockwise distances
      const clockwiseDistance =
        (indexOfNearestEnd - indexOfNearestStart + 64) % 64;

      const counterClockwiseDistance =
        (indexOfNearestStart - indexOfNearestEnd + 64) % 64;

      const clockwiseIsShorter = clockwiseDistance <= counterClockwiseDistance;

      // Collect arc points between start and end
      let arcPoints: [number, number, number][] = [];

      if (clockwiseIsShorter) {
        if (indexOfNearestStart <= indexOfNearestEnd) {
          arcPoints = circlePoints.slice(
            indexOfNearestStart,
            indexOfNearestEnd + 1
          );
        } else {
          // Wrap around
          arcPoints = [
            ...circlePoints.slice(indexOfNearestStart, 64),
            ...circlePoints.slice(0, indexOfNearestEnd + 1),
          ];
        }
      } else if (indexOfNearestStart >= indexOfNearestEnd) {
        for (
          let i = indexOfNearestStart;
          i !== (indexOfNearestEnd - 1 + 64) % 64;
          i = (i - 1 + 64) % 64
        ) {
          arcPoints.push(circlePoints[i]);
        }
      }

      // Continue your custom path logic
      const pathVectors = [
        new THREE.Vector3(start[0], start[1], start[2]), // start
        new THREE.Vector3(raisedStart[0], raisedStart[1], raisedStart[2]), // lift up
        new THREE.Vector3(nearestToStart[0], raisedStart[1], nearestToStart[2]), // move to arc start
        ...arcPoints.map(
          (point) => new THREE.Vector3(point[0], raisedStart[1], point[2])
        ),
        new THREE.Vector3(nearestToEnd[0], raisedEnd[1], nearestToEnd[2]), // move from arc end
        new THREE.Vector3(raisedEnd[0], raisedEnd[1], raisedEnd[2]), // lowered end
        new THREE.Vector3(end[0], end[1], end[2]), // end
      ];

      const customCurve = new THREE.CatmullRomCurve3(
        pathVectors,
        false,
        "centripetal",
        1
      );
      const generatedPoints = customCurve.getPoints(100);
      setCustomCurvePoints(generatedPoints);
    }
  }, [circlePoints, currentPath]);

  // Frame update for animation
  useFrame((_, delta) => {
    if (!ikSolver) return;

    const bone = ikSolver.mesh.skeleton.bones.find(
      (b: any) => b.name === targetBone
    );
    if (!bone) return;

    if (isPlaying) {
      if (!isPaused && customCurvePoints && currentPath.length > 0) {
        const curvePoints = customCurvePoints;
        const speedAdjustedProgress =
          progressRef.current + speed * armBot.speed;
        const index = Math.floor(speedAdjustedProgress);

        if (index >= curvePoints.length) {
          // Reached the end of the curve
          HandleCallback();
          setCurrentPath([]);
          curveRef.current = null;
          progressRef.current = 0;
        } else {
          const point = curvePoints[index];
          bone.position.copy(point);
          progressRef.current = speedAdjustedProgress;
        }
      } else if (isPaused) {
        logStatus(armBot.modelUuid, "Simulation Paused");
      }

      ikSolver.update();
    } else if (!isPlaying && currentPath.length === 0) {
      // Not playing anymore, reset to rest
      bone.position.copy(restPosition);
      ikSolver.update();
    }
  });

  return (
    <>
      {customCurvePoints && currentPath && isPlaying && (
        <mesh rotation={armBot.rotation} position={armBot.position}>
          <Line
            points={customCurvePoints.map(
              (p) => [p.x, p.y, p.z] as [number, number, number]
            )}
            color="green"
            lineWidth={5}
            dashed={false}
          />
        </mesh>
      )}
      <mesh
        position={[
          armBot.position[0],
          armBot.position[1] + 1.5,
          armBot.position[2],
        ]}
        rotation={[-Math.PI / 2, 0, 0]}
        visible={false}
      >
        <ringGeometry args={[1.59, 1.61, 64]} />
        <meshBasicMaterial color="green" side={THREE.DoubleSide} />
      </mesh>
    </>
  );
}

export default RoboticArmAnimator;
